import os
from pylab import *
rc('font', size=12)

amgcl = loadtxt('dmem_data/lrz/amgcl_weak.txt', dtype={
        'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
        'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
        })
trilinos = loadtxt('dmem_data/lrz/trilinos_weak.txt', dtype={
            'names'   : ('mpi', 'size', 'iters', 'setup', 'solve'),
            'formats' : ('i4', 'i8', 'i4', 'f8', 'f8')
            })

omp = unique(list(amgcl['omp']))

def set_ticks():
    gca().set_xscale('log')
    gca().set_xticks([1, 7, 14, 28, 28 * 4, 28 * 16, 28 * 64])
    gca().get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
    gca().get_xaxis().set_tick_params(which='minor', size=0)
    gca().get_xaxis().set_tick_params(which='minor', width=0)

figure(figsize=(8,7))
gs = GridSpec(2,2)
handles = []

subplot(gs[0,0])
for n in omp:
    I = (amgcl['omp'] == n)
    h = loglog(n * amgcl[I]['mpi'], amgcl[I]['setup'] + amgcl[I]['solve'], 'o-')
    handles.append(h[0])
h = loglog(trilinos['mpi'], trilinos['setup'] + trilinos['solve'], 's-')
handles.append(h[0])
set_ticks()
ylim([1e0, 1e2])
ylabel('Total time')

subplot(gs[0,1])
for n in omp:
    I = (amgcl['omp'] == n)
    loglog(n * amgcl[I]['mpi'], amgcl[I]['setup'], 'o-')
loglog(trilinos['mpi'], trilinos['setup'], 's-')
set_ticks()
ylim([1e0, 1e2])
ylabel('Setup time')

subplot(gs[1,0])
for n in omp:
    I = (amgcl['omp'] == n)
    loglog(n * amgcl[I]['mpi'], amgcl[I]['solve'], 'o-')
loglog(trilinos['mpi'], trilinos['solve'], 's-')
set_ticks()
ylim([1e0, 1e2])
ylabel('Solve time')

subplot(gs[1,1])
for n in omp:
    I = (amgcl['omp'] == n)
    semilogx(n * amgcl[I]['mpi'], amgcl[I]['iters'], 'o-')
semilogx(trilinos['mpi'], trilinos['iters'], 's-')
ylim([0, 50])
set_ticks()
ylabel('Iterations')

tight_layout()

figlegend(handles,
    ['AMGCL, omp={}'.format(n) for n in omp] + ['Trilinos'],
    ncol=2, loc='lower center')
gcf().suptitle('Weak scaling of the Poisson problem on the SuperMUC cluster')
gcf().subplots_adjust(top=0.93, bottom=0.15)

show()