from pylab import *
rc('font',   size=12)

def load_data(fname):
    return loadtxt(fname, dtype={
        'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
        'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
        })

def set_ticks(ax, t):
    ax.set_xticks(t)
    ax.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
    ax.get_xaxis().set_tick_params(which='minor', size=0)
    ax.get_xaxis().set_tick_params(which='minor', width=0)

figure(figsize=(8,10))
gs = GridSpec(4,2)
handles = []
omps = set()

for k,fname in enumerate(('dmem_data/mn4/linear_strong.dat', 'dmem_data/mn4/const_strong.dat')):
    data = load_data(fname)
    for omp in sorted(unique(data['omp'])):
        omps.add(omp)

        d = data[data['omp'] == omp]
        c = unique(d['mpi'] * omp)
        m = unique(d['mpi'])

        setup = array([min(d[d['mpi']==i]['setup']) for i in m])
        solve = array([min(d[d['mpi']==i]['solve']) for i in m])
        iters = array([min(d[d['mpi']==i]['iters']) for i in m])
        total = setup + solve

        ax = subplot(gs[0,k])
        h = loglog(c, total, '.-')
        ylim([1e0, 2e2])
        set_ticks(ax, c)
        if k == 0: handles.append(h[0])
        ideal = total[0] * c[0] / c
        if omp == 12:
            hi = plot(c,ideal,'k:', zorder=1, linewidth=1, alpha=0.5)

        ax = subplot(gs[1,k])
        loglog(c, setup, '.-')
        ylim([1e-1, 1e2])
        ideal = setup[0] * c[0] / c
        if omp == 12:
            plot(c,ideal,'k:', zorder=1, linewidth=1, alpha=0.5)
        set_ticks(ax, c)

        ax = subplot(gs[2,k])
        loglog(c, solve, '.-')
        ideal = solve[0] * c[0] / c
        if omp == 12:
            plot(c,ideal,'k:', zorder=1, linewidth=1, alpha=0.5)
        ylim([1e-1, 2e2])
        set_ticks(ax, c)

        ax = subplot(gs[3,k])
        semilogx(c, iters, '.-')
        ylim([0,300])
        set_ticks(ax, c)

    subplot(gs[3,k])
    xlabel('Number of cores (MPI * OMP)')

for fname in ('dmem_data/mn4/trilinos_strong.dat', 'dmem_data/mn4/trilinos_strong_ddml.dat'):
    tri = loadtxt(fname, dtype={
        'names'   : ('mpi', 'size', 'iters', 'setup', 'solve'),
        'formats' : ('i4', 'i8', 'i4', 'f8', 'f8')
        })

    for k in (0,1):
        subplot(gs[0,k])
        h = plot(tri['mpi'], tri['setup'] + tri['solve'], '.-')

        if k == 0: handles += h

        subplot(gs[1,k])
        plot(tri['mpi'], tri['setup'], '.-')

        subplot(gs[2,k])
        plot(tri['mpi'], tri['solve'], '.-')

        subplot(gs[3,k])
        plot(tri['mpi'], tri['iters'], '.-')

subplot(gs[0,0])
title('Linear deflation')
ylabel('Total time')

subplot(gs[0,1])
title('Constant deflation')

subplot(gs[1,0])
ylabel('Setup time')

subplot(gs[2,0])
ylabel('Solve time')

subplot(gs[3,0])
ylabel('Iterations')

figlegend(handles + hi, ['OMP={}'.format(i) for i in sorted(omps)]
        + ['Trilinos/ML', 'Trilinos/DD-ML', 'Ideal scaling'],
       ncol=3, loc='lower center')

tight_layout()

gcf().suptitle('Strong scaling of the Poisson problem on the MareNostrum 4 cluster')
gcf().subplots_adjust(top=0.93, bottom=0.12)

show()